{*
 * File: ...................... direct_addr.pas
 * Autor: ..................... Maxim Artemev aka Bert Raccoon
 * Copyright: ................. (c) 2003 by Max Artemev, MC NTT (www.ntt.ru)
 * Desc: ...................... Support class for direct addresses
 *}
unit direct_addr;

interface
uses Windows, Classes, SysUtils, xml_config, winsock;

type
    TDirectAddr = class
    private
      FAddr: TStringList;
      FPorts: TStringList;

      FFileName: string;
      function CalcNetMask(bits: DWORD): DWORD;
      function GetAddrCount: integer;
      function GetPortCount: integer;
      function GetAddrItem(index: integer): string;
      function GetPortItem(index: integer): string;
    public
      constructor Create(AFileName: string); virtual;
      destructor Destroy; override;
      procedure Load;
      procedure Save;
      procedure Add(ip: string);  overload;
      procedure Add(port: Word); overload;

      procedure Del(ip: string); overload;
      procedure Del(port: Word); overload;

      function IsAddrDirect(IP: DWORD): Boolean;
      function IsPortDirect(Port: WORD): Boolean;

      property AddrCount: integer read GetAddrCount;
      property PortCount: integer read GetPortCount;

      property Addr[index: integer]: string read GetAddrItem;
      property Port[index: integer]: string read GetPortItem;
    end;

    procedure Init;
    procedure Fini;

var
   DirectAddr: TDirectAddr;

implementation


{ TDirectAddr }

procedure TDirectAddr.Add(ip: string);
begin
     FAddr.Add(ip);
end;


procedure TDirectAddr.Add(port: Word);
begin
     FPorts.Add(IntToStr(port));
end;


constructor TDirectAddr.Create(AFileName: string);
begin
     FAddr := TStringList.Create;
     FPorts := TStringList.Create;

     FFileName := AFileName;
     Load;
end;

procedure TDirectAddr.Del(ip: string);
var
   i: integer;
begin
     i := FAddr.IndexOf(ip);
     if i >= 0 then
        FAddr.Delete(i);
end;

procedure TDirectAddr.Del(port: Word);
var
   i: integer;
   port_s: string;
begin
     port_s := IntToStr(port);
     i := FPorts.IndexOf(port_s);
     if i >= 0 then
        FPorts.Delete(i);
end;

destructor TDirectAddr.Destroy;
begin
     if not isLibrary then
        Save;
     FAddr.Free;
     FPorts.Free;
     inherited;
end;


function TDirectAddr.CalcNetMask(bits: DWORD): DWORD;
begin
     result := DWORD((1 shl bits) - 1);
end;


function TDirectAddr.IsAddrDirect(IP: DWORD): Boolean;
var
   direct_ip, direct_netmask, first_ip, last_ip : DWORD;
   s: string;
   i: integer;
begin
     result := False;

     if (IP = 0) or (ntohl(ip) shr 24 = $7F) then
     begin
          result := True;
          exit;
     end;

     for i := 0 to FAddr.Count - 1 do
     begin
          s := FAddr[i];

          if pos('/', s) <> 0 then
          begin
               direct_ip := inet_addr(PChar(copy(s, 1, pos('/', s) - 1)));
               direct_netmask := StrToIntDef(copy(s, pos('/', s) + 1, MaxInt), 0);
               direct_netmask := CalcNetMask(direct_netmask);

               first_ip := direct_ip;
               last_ip := direct_ip or (not direct_netmask);
               result := (ntohl(ip) >= ntohl(first_ip)) and (ntohl(ip) <= ntohl(last_ip));
          end
          else
              result := (ntohl(ip) = ntohl(inet_addr(PChar(s))));

          if result then exit;
     end;
end;

procedure TDirectAddr.Load;
var
   s: string;
   i: integer;
   FIni: TXMLConfig;
begin
     FIni := TXMLConfig.Create(FFileName);
     Fini.ReadSectionValues(PART_DIRECTADDR, 'DirectAddr', FAddr);

     for i := 0 to FAddr.Count - 1 do
     begin
          s := FAddr[i];
          Delete(s, 1, pos('=', s));
          FAddr[i] := s;
     end;

     Fini.ReadSectionValues(PART_DIRECTPORT, 'DirectPort', FPorts);
     for i := 0 to FPorts.Count - 1 do
     begin
          s := FPorts[i];
          Delete(s, 1, pos('=', s));
          FPorts[i] := s;
     end;
     FIni.Free;
end;

procedure TDirectAddr.Save;
var
   i: integer;
   FIni: TXMLConfig;
begin
     FIni := TXMLConfig.Create(FFileName);
     Fini.EraseSection(PART_DIRECTADDR, 'DirectAddr');
     for i := 0 to FAddr.Count - 1 do
       Fini.WriteString(PART_DIRECTADDR, 'DirectAddr', Format('n%d',[i]), FAddr[i]);

     Fini.EraseSection(PART_DIRECTPORT, 'DirectPort');
     for i := 0 to FPorts.Count - 1 do
       Fini.WriteString(PART_DIRECTPORT, 'DirectPort', Format('n%d',[i]), FPorts[i]);
     FIni.Free;
end;


procedure Init;
begin
     DirectAddr := TDirectAddr.Create('freecap.xml');
end;

procedure Fini;
begin
     DirectAddr.Free;
end;


function TDirectAddr.IsPortDirect(Port: WORD): Boolean;
var
   Port_s: string;
   i: integer;
begin
     Port_s := IntToStr(ntohs(Port));
     result := False;

     for i := 0 to FPorts.Count - 1 do
     begin
          result := FPorts[i] = Port_s;
          if result then
             break;
     end;
end;

function TDirectAddr.GetAddrCount: integer;
begin
     result := FAddr.Count;
end;

function TDirectAddr.GetPortCount: integer;
begin
     result := FPorts.Count;
end;

function TDirectAddr.GetAddrItem(index: integer): string;
begin
     result := FAddr[index];
end;

function TDirectAddr.GetPortItem(index: integer): string;
begin
     result := FPorts[index];
end;

end.


